;==========================================================================
;  Anwendung:		TOR2-Verbindung (z.B. mit PC)
;
;	Autor:			E.Mueller
;
;	erstellt:		14.01.2007
;	gendert:		21.03.2008
;
; 	Anpassung ES40 (tiny6k) vpohlers 19.12.2022
;       
;	Prozessor:		U883
;	Takt:				8 MHz
;	geeignet fr:	JU+TE mit 4K Betriebssystem
;
;	Anschlu z.B. an PC
;
;  parallele                JU+TE
;  Schnittstelle            TOR2
;      D0 (2)  <---R100--->  P20
;      D1 (3)  <---R100--->  P21
;      D2 (4)  <---R100--->  P22
;      D3 (5)  <---R100--->  P23
;      D4 (6)  <---R100--->  P24	
;      D5 (7)  <---R100--->  P25
;      D6 (8)  <---R100--->  P26
;      D7 (9)  <---R100--->  P27
;   Masse (0)  <---------->  Masse
;
;==========================================================================
	
;AS-Funktionen
hi              function x,(x>>8)&255
lo              function x, x&255

;------------------------------------------------------------------------------

		cpu	z8601
		assume RP:0C0h		; keine Optimierung !

P2		EQU	02H		; Port 2
p2m 		equ 	0F6h		; Port 2 mode


FLAGS		EQU	0FCH		; Flagregister
RP		EQU	0FDH		; Registerpointer

keybu		equ 	13h		; keypuffer WKEY
KEY		equ	081Bh		; KEY -> 6D
WKEY0		equ	081Eh		; WKEY -> 13
CHAROUT		equ	0818h

buchstabe	MACRO	r8
		ld	15h,#r8
		call	CHAROUT
		ENDM

		;org 02800h	
		org 0F100h
start:		jp	Anfang

;regs:
;40/41	aadr
;42/43	eadr
zaehler		equ	45h
temp		equ	46h
temp_s		equ	47h
;r8	wert
endekenner	equ	49h
;4a/4b		aadr kopie


;Warteschleife
Warte:
		push	rp
		srp	#20h
		ld	r0,#0ffh
   		ld	r1,#0fh		;ggf. erhhen, falls Probleme beim Senden auftreten (Empfang bei PC o..)
wam:
		djnz	r0,wam
   		djnz	r1,wam
		pop	rp	
  		ret

Anfang:
		;Bereich voreinstellen	E000-F1FF
		ld	40h,#0E0h
		ld	41h,#000h
		ld	42h,#hi(start-1)
		ld	43h,#lo(start-1)
Haupt:
		call	Z_Erste
		call	Z_Adresse
		call	Z_Letzte
		call	WKEY
		cp	keybu,#'Z'
		jr	nz,stw1
		ret			;Rckkehr
stw1:
		cp	keybu,#'S'
		jr	nz,stw2
		call	Senden
		jp	Haupt
stw2:
		cp	keybu,#'E'
		jr	nz,stw3
		call	Z_Frage_Bereit
		call	WKEY
		cp	keybu,#'N'
		jp	Z,Haupt
		call	Empfangen
		jp	Haupt
stw3:
		cp	keybu,#'B'
		jp	nz,Haupt
		call	Bereich
		jp	Haupt
	
Senden:
		ld	4ah,40h		;Sicherung oberer Teil Startadresse
		ld	4bh,41h		;Sicherung unterer Teil Startadresse
		ld	p2m,#0		;Tor 2 als Ausgang
		di
		push 	rp
		srp 	#40h
		clr	P2		;auf Null setzen
		call 	Warte
		ld	P2,#0AAh	;Anfang
		call 	Warte
		;********** KOPF ************
		push	02fh
		ld	02fh,40h
		call	Kopf_Adressbyte
		ld	02fh,41h
		call	Kopf_Adressbyte
		ld	2fh,42h
		call	Kopf_Adressbyte
		ld	2fh,43h
		call	Kopf_Adressbyte
		pop	2fh
		ld	P2,#0C3h	;Trenner
		call Warte
		;********** ENDE KOPF ************
		incw	rr2		;Damit auch die letzte Postion kopiert wird !
sew1:
		ldc 	r15,@rr0	;Hole Daten von Adresse 40/41
		and	r15,#0f0h	;oberer Teil
		swap	r15		;tauschen
		add 	r15,#10h	;Kennung oT laden
		ld	P2,r15		;Ausgabe
		call 	Warte
		ldc 	r15,@rr0	;Hole erneut Daten von Adresse
		and	r15,#0fh	;unterer Teil
		add	r15,#20h	;Kennung uT laden
		ld	P2,r15		;Ausgabe
		call 	Warte
		;Weiter
		incw 	rr0
		cp 	r2,r0
		jr 	nz,sew1		;oberer Teil nicht 0, dann weiter
		cp 	r3,r1
		jr 	nz,sew1		;unterer Teil nicht 0, dann weiter
		ld	P2,#55h		;Ende
		call 	Warte
		clr	P2		;auf Null setzen
		call 	Warte
		decw	rr2		;Was man verstellt mu man wieder richten
		pop 	rp
		ei
		ld	p2m,#0ffh	;Tor 2 wieder als Eingang
		ld	40h,4ah		;Wiederherstellung oberer Teil Startadresse
		ld	41h,4bh		;Wiederherstellung unterer Teil Startadresse
		ret

Kopf_Adressbyte:
		ld	4eh,2fh		;arbeite mit siko
		and	4eh,#0f0h	;oberer Teil
		swap	4eh		;tauschen
		ld	4fh,#010h	;Kennung oT laden
		add	4fh,4eh		;Verknpfen
		ld	P2,4fh		;Ausgabe
		call 	Warte
		ld	4eh,2fh		;arbeite mit siko
		and	4eh,#0fh	;unterer Teil
		ld	4fh,#20h	;Kennung uT laden
		add	4fh,4eh		;Verknpfen
		ld	P2,4fh		;Ausgabe
		call 	Warte
		ret	

Empfangen:
		;call	%8dd
		ld	0f6h,#0ffh	;Tor 2 als Eingang
		di
		push 	rp
		srp 	#40h
		;*** Kopf Anfang ***
		;Warte auf Anfang...
Empf_Weiter0:
		;---Abbruchmglichkeit !
		call	KEY
		ld	r8,6dh
		cp	r8,#'A'
		jr	z,Empf_Weiter3
		cp	r8,#'X'
		jr	z,Empf_Weiter3
		;---Abbruchmglichkeit Ende
		cp	P2,#0AAh	;Anfang
		jr	NZ,Empf_Weiter0
		;Weiter Start-,Stopadresse H
		call	Warte_auf_Byte
		ld	40h,r8		;Start		
		ld	42h,r8		;Stop
		;Weiter Start-,Stopadresse L
		call	Warte_auf_Byte
		ld	41h,r8		;Start			
		ld	43h,r8		;Stop
		;Stop -1
		decw	rr2
		;Warte auf Trenner...
Empf_Weiter1:
		;---Abbruchmglichkeit !
		call	KEY
		ld	r8,6dh
		cp	r8,#'A'
		jr	z,Empf_Weiter3
		cp	r8,#'X'
		jr	z,Empf_Weiter3
		;---Abbruchmglichkeit Ende
		cp	P2,#0C3h	;Trenner
		jr	NZ,Empf_Weiter1
		;*** Kopf Ende ***
		;Weiter Daten
Empf_Weiter2:
		call	Warte_auf_Byte
		cp	endekenner,#2	;Ende
		jr	Z,Empf_Weiter3
		incw	rr2
		lde	@rr2,r8
		jr	Empf_Weiter2
Empf_Weiter3:
		pop 	rp
		ei
		ret
	
;Baut 1x und 2x zu einem Byte zusammen
Warte_auf_Byte:
		ld	endekenner,#1
		;---Abbruchmglichkeit !
		call	KEY
		ld	r9,6dh
		cp	r9,#'A'
		jr	z,WaB_EndeZeichen
		cp	r9,#'X'
		jr	z,WaB_EndeZeichen
		;---Abbruchmglichkeit Ende
		;Warte auf 1
		ld	temp_s,P2
		ld	temp,temp_s
		;auf Ende-Zeichen berprfen
		cp	temp,#55h
		jr	Z,WaB_EndeZeichen
		;---
		swap	temp
		and	temp,#0fh
		cp	temp,#1
		jr	NZ,Warte_auf_Byte
		and	temp_s,#0fh
		swap	temp_s
		ld	r8,temp_s
		;Weiter, Warte auf 2
WaB_Weiter:
		;---Abbruchmglichkeit !
		call	KEY
		ld	r9,6dh
		cp	r9,#'A'
		jr	z,WaB_EndeZeichen
		cp	r9,#'X'
		jr	z,WaB_EndeZeichen
		;---Abbruchmglichkeit Ende
		ld	temp_s,P2
		ld	temp,temp_s
		;auf Ende-Zeichen berprfen
		cp	temp,#55h
		jr	Z,WaB_EndeZeichen
		;---
		swap	temp
		and	temp,#0fh
		cp	temp,#2
		jr	NZ,WaB_Weiter
		and	temp_s,#0fh
		add	r8,temp_s
		ret	
WaB_EndeZeichen:
		ld	endekenner,#2
		ret	

Bereich:
		call	Z_Bereich1
		ld	2Ah,#24h	;von
		call	Adresse_eingeben
		buchstabe 0dh		;neue Zeile
		buchstabe 'B'	
		buchstabe 'I'	
		buchstabe 'S'	
		buchstabe ':'	
		ld	2Ah,#26h	;bis
		call	Adresse_eingeben
		call	Z_Bereich2	;bernehmen?
		call	WKEY
		cp	keybu,#'J'
		jr	nz,bew1
		ld	40h,24h
		ld	41h,25h 
		ld	42h,26h
		ld	43h,27h
bew1:
		ret

Z_Adresse:
		srp	#10h		; wg. HTA16
		ld	18h,40h
		ld	19h,41h
		call	00C69h 		; HTA16
		buchstabe '-'	
		ld	18h,42h
		ld	19h,43h
		call	00C69h 		; HTA16
		buchstabe 0dh		;neue Zeile
		buchstabe 0dh		;neue Zeile
		srp	#20h
		ret

Z_Erste:
		push	r13
		push	r14
		push	r15
		ld	r15,#lo (Z_Erste_Bild)
		ld	r14,#hi (Z_Erste_Bild)
		call	Baue_Bild
		pop	r15
		pop	r14
		pop	r13
		ret
	
Z_Letzte:
		push	r13
		push	r14
		push	r15
		ld	r15,#lo (Z_Letzte_Bild)
		ld	r14,#hi (Z_Letzte_Bild)
		call	Baue_Bild
		pop	r15
		pop	r14
		pop	r13
		ret

Z_Frage_Bereit:
		push	r13
		push	r14
		push	r15
		ld	r15,#lo (Z_Frage_Bild)
		ld	r14,#hi (Z_Frage_Bild)
		call	Baue_Bild
		pop	r15
		pop	r14
		pop	r13
		ret

Z_Bereich1:
		push	r13
		push	r14
		push	r15
		ld	r15,#lo (Z_Bereich1_Bild)
		ld	r14,#hi (Z_Bereich1_Bild)
		call	Baue_Bild
		pop	r15
		pop	r14
		pop	r13
		ret

Z_Bereich2:
		push	r13
		push	r14
		push	r15
		ld	r15,#lo (Z_Bereich2_Bild)
		ld	r14,#hi (Z_Bereich2_Bild)
		call	Baue_Bild
		pop	r15
		pop	r14
		pop	r13
		ret

Baue_Bild:
		ldc	r13,@rr14
		cp	r13,#01
		jr	z,BB_W1
		cp	r13,#02
		jr	z,BB_W2
		cp	r13,#03
		jr	z,BB_W3
		;normales Zeichen
		ld	15h,r13
		call	CHAROUT		; Ausgabe
BB_W0:
		incw	rr14	
		jr	Baue_Bild
BB_W1:
		buchstabe 0ch		;CLS Bildschirm lschen
		jr	BB_W0
BB_W2:
		buchstabe 0dh		;neue Zeile
		jr	BB_W0
BB_W3:
		ret

Z_Frage_Bild:	
		;01 Bildschirm lschen
		;02 neue Zeile
		;03 Ende
		db	01
		db "BILD WIRD NUN"
		db "ABGESCHALTET!"
		db	02
		db "ABBRUCH DURCH"
		db "TASTE A BZW X"
		db	02
		db "BEREIT(J/N)?"
		db 03

Z_Erste_Bild:
		db	01
		db "TOR2-VERBIND."
		db	02
		db "B BEREICH"
		db	02,03

Z_Letzte_Bild:
		db "E EMPFANGEN"
		db	02
		db "S SENDEN"
		db 02
		db "Z ZURUECK "
		db 03

Z_Bereich1_Bild:
		db	01
		db "BEREICH"
		db	02
		db "FESTLEGEN"
		db	02,02
		db "VON:"
		db	03
	
Z_Bereich2_Bild:
		db	02,02
		db "UEBERNEHMEN?"
		db	03

Taste_erkannt:
		ld	15h,22h		;Darstellen	
		call	CHAROUT
		ld	@2Fh,20h	;Lade Zeichen in Zwischenspeicher
		dec	2Fh		;Stellenzeiger Zwischenspeicher
		srp	#20h
		djnz	r3,Tastenabfrage
		rl	2Eh		;1.Stelle
		rl	2Eh
		rl	2Eh
		rl	2Eh
		and	2Eh,#0F0h
		rl	2Ch		;3.Stelle
		rl	2Ch
		rl	2Ch
		rl	2Ch
		and	2Ch,#0F0h
		add	2Eh,2Dh		;1.& 2.Stelle zusammenfhren
		add	2Ch,2Bh		;3.& 4.Stelle zusammenfhren
		ld	@2Ah,2Eh	;oberer Adressteil
		inc	2Ah
		ld	@2Ah,2Ch	;unterer Adressteil
		ret	

Adresse_eingeben:
		ld	23h,#04h	;Stellenzhler auf 4
		ld	2Fh,#2Eh	;Stellenzeiger
Tastenabfrage:
		push	r13
		push	r14
		push	r15
Erl_Tast_W0:
		ld	r15,#lo (Erlaubte_Tasten)
		ld	r14,#hi (Erlaubte_Tasten)
		call	WKEY0
		ld	22h,keybu
		clr	20h
Erl_Tast_W1:
		ldc	r13,@rr14
		cp	22h,r13
		jr	Z,Erl_Tast_W2
		inc	20h
		cp	20h,#16
		jr	Z,Erl_Tast_W0
		incw	rr14
		jr	Erl_Tast_W1
Erl_Tast_W2:
		pop	r15
		pop	r14
		pop	r13
		jp	Taste_erkannt

Erlaubte_Tasten:	
		db "0123456789ABCDEF"

;WKEY mit Echo
WKEY:		call	WKEY0
		ld	15h,13h
		call	CHAROUT
		ret
	
		end
;M
;JF200
