;------------------------------------------------------------------------------
; TINY ES40 EPROM-Bank
; (c) V. Pohlers 2021
;------------------------------------------------------------------------------
; Bank-Steuersoftware, Bank 0
; wird im ES 40c mit Mon X gestartet
; 26.08.2021, letzte nderung 25.04.2022
;------------------------------------------------------------------------------


;AS-Funktionen
hi              function x,(x>>8)&255
lo              function x, x&255

;------------------------------------------------------------------------------

		cpu	z8601


P3		EQU	03H		; Port 3
P3BIT		EQU	7		; Schaltbit 3.7
;P3BIT		EQU	4		; Schaltbit 3.4 bei Hardware nach
					; https://github.com/seidat1/Tiny Rombank

P3MASK		EQU	1 << P3BIT

FLAGS		EQU	0FCH		; Flagregister
RP		EQU	0FDH		; Registerpointer
SPH		EQU	0FEH		; Stackpointer, Highteil
SPL		EQU	0FFH		; Stackpointer, Lowteil


; ES40c-Routinen
setbank		equ	92Fh		; Bank setzen in ES 40c

KOMMAND		equ	0812h
CHARIN        	equ	0815h
CHAROUT       	equ	0818h
KEY           	equ	081Bh
WKEY          	equ	081Eh
SAVE          	equ	0821h
LOAD          	equ	0824h
SCRFUN        	equ	0827h		; screen functions
MONITOR       	equ	082Ah
PRISTRI       	equ	082Dh		; Textausgabe
SHOWPLAYER    	equ	0830h
HIDEPLAYER    	equ	0833h
RND           	equ	0836h

HTA16		equ	0C69h		; Ausgabe %18/19 als 4-Ziffer-Hexzahl
HTA8		equ	0C72h		; Ausgabe %19 als 2-Ziffer-Hexzahl

var_X_lo	equ 	4Fh
var_Y_lo	equ 	51h
var_Z_lo	equ 	53h

at	macro zeile,spalte
    		LD      var_Y_lo,#zeile
		LD      var_X_lo,#spalte
    		LD      var_Z_lo,#1   ;Kursor setzen
    		CALL    SCRFUN
	endm

; max 100h Platz fr die RAM-Prog. ggf. im Listing berwachen!

;ramprog		equ	0F400h		; wo ist das am besten aufgehoben ??? 100h Platz reicht
; 	f400 geht leider nicht :-(
;ramprog		equ	0Bf00h
;ramprog		equ	0CF00h		; ok
;ramprog		equ	09F00h		; wg. MINE 74
ramprog			equ	0F57Bh		; Kassettenpuffer + Stack
						; wg .steine

; ROM-Bank Eigenschaften
bankstart	equ	2000h
blocksize	equ	2000h	; 8K
lastbank	equ	00fh	; 16 Bnke
systembank	equ	0

;------------------------------------------------------------------------------

		org	02000h
		assume RP:0C0h			; keine Optimierung !

		; sprungverteiler
		jp	main			; Hauptprogramm, Start mit Mon X
		jp	3000h			; baserw
		jp	search0			; Suchroutine umladen
		jp	load0			; Lade+Entpackroutine umladen
		jp	ramprog			; Routine starten
		jp	start_prog		; Programm starten
		db	"VP6"			; Kennung fr Banksuche0
		jp	float			; Flot-Erweiterung in RAM umladen

;------------------------------------------------------------------------------
; Anzeige ROM-Bankinhalt
; Auswahl eines Programms und Start

main:		; einstieg mit 14/15 = F701 2. eing. Zeichen oder ' '
		; folgt nach Kommando "X"  eine Programmnummer?
		srp	#10h
		ld	r0,#16h
		ld	r1,#6			; max 6 zeichen
mm1:		ldei	@r0,@rr14		; in BASIC-Buffer umlagern
		djnz	r1, mm1
		call	1D9h			; BASIC: number
		ld	2Ah,r5			; konvertierte Zahl (lo byte)

		srp	#20h
		cp	r10, #0			; r10 = Zahl oder 0
		jr	nz, main1		; direkte Suche

		; sonst DIR
main0:		srp	#20h
		call	dir0			; Umladen
		call	ramprog			; und DIR starten

		; Eingabe verarbeiten
main2:		call	numin
		cp	r10, #0
		jr 	nz, main1		; gltige Zahl

		;sonst Rcksprung in Monitor
		;;ld	15h,13h			; M anzeigen
		;;call	CHAROUT			;; nicht ntig, es kommt der Text "Mon"
		ld	15h,#0dh
    		call	CHAROUT
		ret				; ENDE

main1:		call	search0			; Umladen 
		call	ramprog			; und search starten

; out: 20h = 0 -> nicht gefunden
; 	2Bh (r11) bank
;	2Ch (r12) Hi Adr

		cp	r10, #0
		jr	nz,main3
		; nicht gefunden
		ld	15h,#'?'
    		call	CHAROUT			; Ausgabe Fehler
		jr	main2			; neue Eingabe

main3:
		if 1=0
		; Anzeige Bank und Adresse
		srp 	#10h
    		ld	15h,#' '
    		call	CHAROUT
		ld	19h, 2Bh
		call	HTA8
    		ld	15h,#' '
    		call	CHAROUT
		ld	18h, 2Ch
		ld	19h,#0
		call	HTA16
		srp 	#20h
		endif

		;load
		call	load0			; Umladen
		call	ramprog			; und starten

		; jetzt rp = 10h !!
		; in 6/7 steht die Startadr

		; starte Programm
start_prog:	cp	r10,#'M'
		jr	eq, start_mc

		cp	r10,#'P'
		jr	eq, start_bmp

		cp	r10,#'B'
		jr	nz,mainret	; sonstige Typen nur laden, kein start
		
		
spsav		equ	0F79Eh
		
		; starte BASIC
start_basic:	; SP sichern
		ld	R0,#hi(spsav)
		ld	R1,#lo(spsav)
		ld	R2,#SPH
		ldei	@RR0,@R2
		ldei	@RR0,@R2

		CLR 8 			; keine Proctab
		CLR 9			; in 6/7 = Adr Basic-Pgm (Std. E000)
		
		call  07FDh 		; Interpreter starten
		; kein jp basic, da Stackpointer durch gosub etc nich passen muss
		
		; SP wiederherstellen
		di
		ld	R0,#hi(spsav)
		ld	R1,#lo(spsav)
		ld	R2,#SPH
		ldei	@R2,@RR0
		ldei	@R2,@RR0
		ei
		ret		

start_mc:	; starte MC

		cp	6,#0
		jr	eq, mainret	; kein Autostart
		jp	@6
mainret:	; 22.04.2022 Basic-Adresse wieder setzen (E000)
		clr	8
		ld	7, #00h
		ld	6, #0E0h
		ret

		; Starte Bildbetrachter (todo)

start_bmp:	;CALL	PRISTRI
		;db	"BMP-Viewer fehlt noch",0dh,0
		;ret
		jp	bmpview

		include	"Software/Rolf/Bildbetrachter/bildbetrachter.asm"

;------------------------------------------------------------------------------
; Umladeroutinen
; whrend Aktionen mit der ROM-Bank ist dieser Codebereich nicht nutzbar,
; da diese Bank weggeschaltet wird, deshalb Umladen in RAM (-> ramprog) 

dir0:		ld	r0,#hi(diranf)
		ld	r1,#lo(diranf)
		ld	r4,#hi(dirlen)
		ld	r5,#lo(dirlen)
		jp	ldir			; Umladen und DIR starten

search0:	ld	r0,#hi(searchanf)
		ld	r1,#lo(searchanf)
		ld	r4,#hi(searchlen)
		ld	r5,#lo(searchlen)
		jp	ldir			; Umladen und search starten

load0:		ld	r0,#hi(loadanf)
		ld	r1,#lo(loadanf)
		ld	r4,#hi(loadlen)
		ld	r5,#lo(loadlen)
		jp	ldir			; Umladen und load starten


;------------------------------------------------------------------------------
; Zahleneneingabe
; ret	r10: nummer
numin:		srp	#10h
		and	55h, #0fbh	; Simulationsmode, Bit 2 von Register %55 null
		call	2E9h		; Input vom BASIC
		or	55h, #04h	; Simulationsmode aufheben
		srp	#20h
		ld	r10, 13h	; rr2 enthlt zahl
		ret

;------------------------------------------------------------------------------
; Bereich nach ramprog verschieben + starten
; in rr0 quelle, rr4 laenge

ldir:		ld	r2,#hi(ramprog)
		ld	r3,#lo(ramprog)
mov1:		lde	r6,@rr0
		lde	@rr2,r6
		incw	rr0
		incw	rr2
		decw	rr4
		jr	nz, mov1
		;jp	ramprog		; starten
		ret

;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
; DIR
;------------------------------------------------------------------------------

diranf		equ	$

		phase	ramprog

dir:		srp	#20h

    		CALL	PRISTRI
    		db	0Ch		; cls
    		db	"EPROMBANK    (c) VPohlers 08/2021",0dh,0dh
    		db	0

		ld	54h, #0		;systembank
		call	setbank
		ld	r13,#0		;lfd nr



		ld	r12,#0		;lfd nr Anzeige
		ld	r14, #hi(bankstart)
dir3:		clr	r15		; xx00-Adresse
		lde	r5,@rr14
		cp	r5,#0fah
		jr	nz,dir2
		incw	rr14
		lde	r5,@rr14
		cp	r5,#0fah
		jr	nz,dir2
		; Kennung gefunden
		call	show

		cp	r12,#36		; max 36 Eintrge auf einmal anzeigen
		jr	lt,dir2

		call	WKEY		;Pause, warten auf taste
		ld	r12,#0		;Zhler wieder auf 0 setzen

dir2:		;weitersuchen
		inc	r14
		cp	r14, #hi(bankstart+blocksize)
		jr	ult,dir3
		;next bank
		ld	r14,#hi(bankstart)	; zurck auf 2000h
		clr	r15
		AND 	P3, #(~P3MASK & 0FFh)
		OR 	P3, #P3MASK	;Impuls an P3BIT: Zhler weiter
		inc	54h
		cp	54h,#lastbank
		jr	ule, dir3

dir4:		; Ende
    		;;;at 21,0
    		CALL	PRISTRI
    		db	0dh
    		DB	"Nr waehlen, 0-Monitor: ",0

		;;;ld	r10,r14		;aktuelle Pos sichern
		;;;ld	r11,54h		;aktuelle Bank sichern

		ld	54h, #0		;systembank
		call	setbank
		ret

show:  		add	r13,#1		; lfd Nr erhhen
		da	r13		; dezimal
		inc	r12

		ld	19h,r13
    		srp	#10h
    		call	HTA8		; lfd Nr anzeigen
    		srp	#20h
    		ld	15h,#' '
    		call	CHAROUT
    		ld	r15, #15	; offs. Typ
    		lde	r5,@rr14
    		ld	15h,r5
    		call	CHAROUT
    		ld	15h,#' '
    		call	CHAROUT

   		; Anzeige Programmname
    		ld	r1,#14		; Name 16 zeichen,  aber
    		; nur 14 anzeigen, dann passen genau 2 Namen pro Zeile

    		ld	r15,#10h	; offs. Name
show1:    	lde	r5,@rr14
		ld	15h,r5
    		call	CHAROUT
    		incw	rr14
   		djnz	r1, show1
    		ld	15h,#' '
    		call	CHAROUT
		ret

		dephase

dirlen		equ	$-diranf

;------------------------------------------------------------------------------
; search
; in: 2Ah  (r10) gesuchte Nummer
; out: 2Ah = 0 -> nicht gefunden
; 	2Bh (r11) bank
;	2Ch (r12) Hi Adr
;------------------------------------------------------------------------------

searchanf	equ	$

		phase	ramprog

search:		srp	#20h
		ld	r13,#0		;lfd nr
		ld	r14, #hi(bankstart)
search3:	clr	r15		; xx00-Adresse
		lde	r5,@rr14
		cp	r5,#0fah
		jr	nz,search2
		incw	rr14
		lde	r5,@rr14
		cp	r5,#0fah
		jr	nz,search2
		; Kennung gefunden
		inc	r13
		cp	r13,r10		; vergleich mit gesuchter nummer
		jr	z, search4	; gefunden
search2:	;weitersuchen
		inc	r14
		cp	r14, #hi(bankstart+blocksize)
		jr	ult,search3
		;next bank
		ld	r14,#hi(bankstart)	; zurck auf 2000h
		clr	r15
		AND 	P3, #(~P3MASK & 0FFh)
		OR 	P3, #P3MASK	;Impuls an P3BIT: Zhler weiter
		inc	54h
		cp	54h,#lastbank
		jr	ule, search3
		ld	r10,#0		; alles durchsucht, aber nicht gefunden
search4:	; Ende
		ld	r11,54h		; aktuelle Bank
		ld	r12,r14		; akuelle Pos

		ld	54h, #0		;systembank
		call	setbank
		ret

		dephase
searchlen	equ	$-searchanf

;------------------------------------------------------------------------------
; load
; 	2Bh (r11) bank
;	2Ch (r12) Hi Adr
; ret: 	RP = 10h (!)
; 	r10 = type
; 	6/7 = sadr
;------------------------------------------------------------------------------

loadanf		equ	$
		phase	ramprog

		srp 	#10h
        	
		ld	54h, 2Bh	;#prgbank
		call	setbank
		ld	r4, 2Ch		;hi(prgpos), lo ist immer xx00h
;       	
		ld	r5,#6		; offset sadr
		lde	r10,@rr4
		incw	rr4
		lde	r11,@rr4
		push	r10		; save saddr
		push	r11
        	
		ld	r5,#15		; offset type
		lde	r6,@rr4
		push	r6		; save type
        	
        	
		; Quelladresse rr0
		ld	r0,r4		; hi pos
		ld	r1,#20h		; lo pos ist immer xx00h + headersize = 20h
        	
		; Zieladresse rr2
		ld	r5,#2		; offs. aadr
		lde	r2,@rr4		; hi aadr
		incw	rr4
		lde	r3,@rr4		; lo aadr
        	
		ld	r5,#8		; offs. packed
		lde	r6,@rr4
		tm	r6,#80h
		jr	z, copy1
		; unpack
		call	depack
		jr	copy2
        	
copy1:		; einfaches ldir
		ld	r5,#4		;offs. eadr
		lde	r6,@rr4
		incw	rr4
		lde	r7,@rr4
		sub	r7,r3
		sbc	r6,r2		; rr6 = laenge
        	
copy1a: 	
		call	ldi		; copy from rr0 to rr2
		decw	rr6
		jr	nz, copy1a
        	
copy2:		ld	54h, #0
		call	setbank		; Bank 0
        	
		pop	r10		; r10 = type
		pop	7
		pop	6		; 6 = sadr
        	
        	
		ret

		include	depack_z8b.asm

		dephase
loadlen		equ	$-loadanf

;------------------------------------------------------------------------------
; 25.04.2022 Float-Erweiterung in RAM umladen
; 10 CALL %2015:REM Float laden
; 20 Call %DAFA:REM Float Init.
; 40l[0]=0.42
; 50?[0]
;------------------------------------------------------------------------------

	include	packedrom.inc

float:		;load
		srp	#10h
		push	r0			; save basic-pointer
		push	r1
		
		ld	2bh, #bank_float
		ld	2ch, #hi(pos_float)
		call	load0			; Umladen
		call	ramprog			; und starten
		
		srp	#10h
		pop	r1			; restore
		pop	r0
		jp	mainret			; dito reset Reg6/7

;------------------------------------------------------------------------------

	end
