;------------------------------------------------------------------------------
; TINY ES40 EPROM-Bank
; (c) V. Pohlers 2021
;-----------------------------------------------------------------------------
; bitbuster extreme depack
; Z8-Version V.Pohlers 23.08.2021 / 26.08.2021 11:28:40
;------------------------------------------------------------------------------
; translated from original depack.gen
; rr0 hl
; rr2 de
; r4  a
; r5
; rr6 bc
; rr8 hl'		'
; rr10 de'		'
; r12 b'		'
;------------------------------------------------------------------------------

; version 3 mit extra UP getbyte, funktioniert
; mit Bankwechsel

;In: 	RP 	z.B. %10
;	RR0	source
;    	RR2	destination
; 	
depack: 	ld	r4,#80h
		clr	r10			;RR10=#1
		ld	r11,#1
	
depack_loop:	call	getbit			;get compression type bit
		jr	c,output_compressed	;if set, we got lz77 compression
		call	ldi			;copy byte from compressed data to destination (literal byte)
		jr	depack_loop
	
;handle compressed data
output_compressed:
		call	getbyte
		ld	r7,r5			;get lowest 7 bits of offset, plus offset extension bit

output_match:	clr	r6
		tm	r7,#80h
		jr	z,output_match1		;no need to get extra bits if carry not set
        	
		call	getbit			;get offset bit 10 
		rlc	r6
		call	getbit			;get offset bit 9
		rlc	r6
		call	getbit			;get offset bit 8
		rlc	r6
		call	getbit			;get offset bit 7
        	
		jr	c,output_match1		;since extension mark already makes bit 7 set 
		and	r7,#7fh			;only clear it if the bit should be cleared
output_match1:	incw	rr6
		
;return a gamma-encoded value
;length returned in rr8
		ld	r8,r10
		ld	r9,r11         	 	;initial length to 1
		ld	r12,r11			;bitcount to 1
	
;determine number of bits used to encode value
get_gamma_value_size:
		call	getbit			;get more bits
		jr	nc,get_gamma_value_size_end	;if bit not set, bitlength of remaining is known
		inc	r12			;increase bitcount
		jr	get_gamma_value_size	;repeat...

get_gamma_value_bits:
		call	getbit			;get next bit of value from bitstream
		adc	r9,r9			;;insert new bit in HL
		adc	r8,r8
	
get_gamma_value_size_end:
		djnz	r12,get_gamma_value_bits	;repeat if more bits to go

get_gamma_value_end:
		incw	rr8			;length was stored as length-2 so correct this
		jr	nc, g2			;rr8 = length
		ret
        	
g2		ld	r14,r2
		ld	r15,r3			;destination address in HL...
		sub	r15,r7			;calculate source address
		sbc	r14,r6
        	
g1:		;ldir, copy byte from decompressed data to destination
		;hier ohne Banktest, die Bytes sind schon entpackt, Quelle ist normaler RAM
		lde	r5,@rr14		;get 8 bits from bitstream
		lde	@rr2,r5
		incw	rr14	
		incw	rr2
		decw	rr8			;Match length
		jr	nz, g1
        	
		jr	depack_loop
	
;UP: get a bit from the bitstream
;carry if bit is set, nocarry if bit is clear
;must be entered with second registerset switched in!
getbit:		add	r4,r4			;shift out new bit
		jp	nz,getbit1		;if remaining value isn't zero, we're done
        	
        	call	getbyte
        	ld	r4,r5
		rlc	r4			;(bit 0 will be set!!!!)
getbit1:	ret

;UP: transfer from rr0 to rr2
ldi:		call	getbyte			;copy byte from compressed data to destination (literal byte)
		lde	@rr2,r5
		incw	rr2
		ret

;;bankstart	equ	2000h	; 8K
;;blocksize	equ	2000h	; 8K

;UP: get next byte
getbyte:	push	flags
		lde	r5,@rr0			;get 8 bits from bitstream
		incw	rr0			;increase source data address
		;ggf hier bankumschaltung
		cp	r0,#hi(bankstart+blocksize)	; adresse 4000 erreicht?
		jr	ult,getbyte1	

		;next bank
		AND 	P3, #7Fh	
		OR 	P3, #80h	;Impuls an P3.7: Zhler weiter

		ld	r0,#hi(bankstart)	; zurck auf 2000h
		clr	r1
		
getbyte1:	pop	flags
		ret
		