;------------------------------------------------------------------------------
; TINY ES40 EPROM-Bank
; (c) V. Pohlers 2021
;------------------------------------------------------------------------------
; Zusammenstellung Bankinhalt, erzeugt den ROM
; 21.08.2021, letzte nderung 
;------------------------------------------------------------------------------
; bei nderungen in dieser Datei unbedingt 'make depend' starten!
;------------------------------------------------------------------------------

;        cpu 96C141			; Mikrocontroller mit groem Adressbereich
;        maxmode on			; der Z8 reicht hier nicht.
					; leider little endian
	
	cpu 80C251			; geht mit bigendian :-)
        BIGENDIAN on

	MACEXP  off

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -

;AS-Funktionen
hi              function x,(x>>8)&255
lo              function x, x&255
x_align	function x, (($+x-1)/x)*x

myalign	macro	size
	org	x_align(size)
	endm

bankstart	equ	2000h
blocksize	equ	2000h	; 8K
lastbank	equ	00fh	; 
systembank	equ	0

; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
; MACRO a d d f i l e
;
	;addFile filename, aadr, [eadr], [sadr], typ, name, kommentar
;
;filename mit Pfad
;eadr kann bei ungepackten Daten leer bleiben (dann = aadr+filelength)
;sadr kann leer bleiben (dann = aadr)
;typ = Dateityp (B = Tiny-MPBASIC, M = MC-Binary)
;Name = xx Zeichen
;Kommentar = xx 13 Zeichen
;
; align	100h verschwendet etwas Speicher, aber ist vermutlich leichter zu handlen
; fileXname als Dummy anlegen

addFile	macro	fileXname,aadr,eadr,sadr,typ,name

		;align	20h	
		align	100h	

bank 	eval 	($ / blocksize)
pos	eval 	($ # blocksize + bankstart)

	if	"eadr" <>""
eaddr	eval	eadr
	else
eaddr	eval	aadr+file_bis-file_von
	endif
	if	"sadr"<>""
saddr	eval	sadr
	else
saddr	eval	aadr
	endif

len 	eval	eaddr-aadr+1

packed	eval	0
	if STRSTR(fileXname,".pck") > 0
packed	eval	80h
eaddr 	eval	0
	endif
	
		; Header
		db	0FAH,0FAH			; 2 magic marker FlAsH
		dw	aadr, eaddr, saddr		; 6
		;;db	hi(aadr), lo(aadr)
		;;db	hi(eaddr), lo(eaddr)
		;;db	hi(saddr), lo(saddr)
		db	packed
		db	6 dup(0)			; reserviert
		db	typ				; 1
		db	substr(name+"                ",0,16)	; 16

		; file
		align	20h			; align knnte entfallen; Header ist 20h lang
file_von
		BINCLUDE  fileXname
file_bis

xeaddr	eval	aadr+file_bis-file_von

		MESSAGE "\{bank}, \{pos}h, aadr, \{eaddr}h [\{xeaddr}h], \{saddr}h, typ, "+name+", \{len}, "+fileXname

		endm

;------------------------------------------------------------------------------

	;bank0
	
	binclude	bank0.bin
	org	1000h			; offs 3000h
	binclude	"Software/volker/baserw40.bin"

	; wg. Kompatibilitt zur orig. ROM-Bank
	; CALL %A4A -> CALL 37E0 laden Z1013-ZS
	org	17D0h			; offs 37D0h
	binclude	"zg.bin"


;------------------------------------------------------------------------------

	align	blocksize

	;addFile filename, aadr, [eadr], [sadr], typ, name
	;Packen - Endung pck ergnzen!

	;bei jtc -> include file.bin.inc ;; pck funktioniert dann nicht ???

	addFile "Software/kassette6k/asdis_6k.bin.pck"		, 0C100h, 0C9EFh, 0, 'M', "ASDIS" 
	addFile "Software/kassette6k/bitman_6k.bin.pck"		, 0C000h, 0C05Fh, 0, 'M', "Bitman" 
	addFile "Software/kassette6k/kctrans_6k.bin.pck"	, 0F100h, 0F3FFh, 0, 'M', "KC-Trans" 
	addFile "Software/kassette6k/eprommer_6k.bin.pck"	, 0E000h, 0EFFFh, , 'B', "EPROM-Br.6k" 	; Basic + MC dahinter
	addFile "Software/kassette6k/float_fuer_basic.bin.pck"	, 0D000h, 0DFFFh, , 'M', "BasicFloat" 
	shared 	bank_float, pos_float
bank_float 	equ	bank
pos_float	equ	pos
	
;;	addFile "Software/kassette6k/forth_6k.bin.pck"		, 0D000h, 0E4FFh, , 'M', "Forth 6k" 
	addFile "Software/volker/forth_6k_vp_d000.bin.pck"	, 0D000h, 0E4FFh, , 'M', "Forth 6k" 
	; liegt als ASM zg_z9001.asm vor
;;	addFile "Software/volker/zg_z9001.bin.pck"		, 0B000h, , 0B800h, 'M', "ZG Z9001" 
	addFile "Software/volker/zg_z9001.bin"			, 0C0F0h, , ,  'M', "ZG Z9001" 

	addFile "Software/kassette6k/hase_wolf_6k.bin.pck"	, 0E000h, 0E5F1h, , 'B', "HaseWolf 6k" 
	addFile "Software/kassette6k/kassetteninhalt_6k.bin.pck", 0E000h, 0E61Dh, , 'B', "Kass-Info" 
	addFile "Software/kassette6k/master_mind_6k.bin.pck"	, 0E000h, 0E3CDh, , 'B', "Master M.6k" 
	addFile "Software/kassette6k/s3004_treiber.bin.pck"	, 0F480h, 0F57Fh, 0, 'M', "S3004-Treib" 
	
	addfile "Software/Perser/Apollo_E000.BIN.pck"           , 0E000h, , , 'B', "Apollo"

	addfile "Software/Rolf/Soft_8000/HANOI_8.bin.pck"       , 08000h, , , 'M', "HANOI"
	addfile "Software/Rolf/Soft_8000/LADDER_8.bin.pck"      , 08000h, , , 'M', "LADDER"
	addfile "Software/Rolf/Soft_8000/LIFE_8.bin.pck"        , 08000h, , , 'M', "LIFE"
	addfile "Software/Rolf/Soft_8000/MERKS_8.bin.pck"       , 08000h, , , 'M', "MERKS"
	addfile "Software/Rolf/Soft_8000/NIMM_8.bin.pck"        , 08000h, , , 'M', "NIMM"
	addfile "Software/Rolf/Soft_8000/SCHIEB_8.bin.pck"      , 08000h, , , 'M', "SCHIEB AB"
	addfile "Software/Rolf/Soft_8000/TTT_8.bin.pck"         , 08000h, , , 'M', "TIC-TAC-TOE"
	addfile "Software/Rolf/Soft_8000/UHR_8.bin.pck"         , 08000h, , , 'M', "UHR"
	addfile "Software/Rolf/Soft_8000/Wurmi_8.bin.pck"       , 08000h, , , 'M', "Wurmi"
	;2000h geht leider nicht, da im ROM liegend
	;addfile "Software/Rolf/Testbild/testbild_2000.bin.pck"  , 02000h, , , 'M', "Testbild"
	;deshalb als ASM
	addfile "Software/Rolf/Testbild/testbild.bin.pck"  	, 08000h, , , 'M', "Testbild"
	addfile "Software/Thomas/MINE_8_korr.bin.pck"           , 08000h, , , 'M', "MINE"
	addfile "Software/Thomas/SOKOBAN_8_korr.bin.pck"        , 08000h, , , 'M', "SOKOBAN"
	;addfile "Software/Thomas/Steine_8_korr.bin.pck"         , 08000h, , , 'M', "Steine"
	;Wolfgang 15.09.2021 neu
	addfile "Software/Thomas/6k-steine_Kursor.bin.pck"      , 08000h, 09FFFh, , 'M', "Steine"
	addfile "Software/Thomas/TETRIS_8_korr.bin.pck"         , 08000h, , , 'M', "TETRIS"

	addfile "Software/Wolfgang/17UND4.bin.pck"              , 0E000h, , , 'B', "17UND4"
;;23.12.2021 auf Wunsch von Wolfgang wieder raus
;;	addfile "Software/Wolfgang/ALIEN.bin.pck"               , 0E000h, , , 'B', "ALIEN"
	addfile "Software/Wolfgang/ANGELN_neu.bin.pck"          , 0E000h, , , 'B', "ANGELN"
	addfile "Software/Wolfgang/AUTOFAHRT.bin.pck"           , 0E000h, , , 'B', "AUTOFAHRT"
	addfile "Software/Wolfgang/BOMBER.bin.pck"              , 0E000h, , , 'B', "BOMBER"
	addfile "Software/Wolfgang/BONK.bin.pck"                , 0E000h, , , 'B', "BONK"
	addfile "Software/Wolfgang/BOWLING.bin.pck"             , 0E000h, , , 'B', "BOWLING"
	addfile "Software/Wolfgang/BRUECKE.bin.pck"             , 0E000h, , , 'B', "BRUECKE"
	addfile "Software/Wolfgang/COLORMA.bin.pck"             , 0E000h, , , 'B', "COLOR-MASTERMIND"
	;testweise unpacked
	;addfile "Software/Wolfgang/COLORMA.bin"                 , 0E000h, , , 'B', "COLOR-MASTERMIND"
	addfile "Software/Wolfgang/ENTENJAGD.bin.pck"           , 0E000h, , , 'B', "ENTENJAGD"
;;23.12.2021 auf Wunsch von Wolfgang wieder raus
;;	addfile "Software/Wolfgang/FLIEGER.bin.pck"             , 0E000h, , , 'B', "FLIEGER"
	;addfile "Software/Wolfgang/FROSCH.bin.pck"              , 0E000h, , , 'B', "FROSCH"
	;Wolfgang 15.09.2021 neu
	addfile "Software/Wolfgang/FROSCH_neu.bin.pck"          , 0E000h, , , 'B', "FROSCH"
	addfile "Software/Wolfgang/JUMPER.bin.pck"              , 0E000h, , , 'B', "JUMPER"
	addfile "Software/Wolfgang/LANDERV6.bin.pck"            , 0E000h, , , 'B', "LANDER"
	addfile "Software/Wolfgang/OPA_SINCLAIR.bin.pck"        , 0E000h, , , 'B', "OPA SINCLAIR"
	addfile "Software/Wolfgang/PAUL.bin.pck"                , 0E000h, , , 'B', "PAUL"
	addfile "Software/Wolfgang/ROEMISCH.bin.pck"            , 0E000h, , , 'B', "ROEMISCH"
	addfile "Software/Wolfgang/SKARA.bin.pck"               , 0E000h, , , 'B', "SKARABAEUS"
	addfile "Software/Wolfgang/SKEET.bin.pck"               , 0E000h, , , 'B', "SKEET"
	addfile "Software/Wolfgang/WOCHENTG.bin.pck"            , 0E000h, , , 'B', "WOCHENTAG"
	addfile "Software/Wolfgang/WURF.bin.pck"		, 0E000h, , , 'B', "WUERFELN"
	;27.08.2021 15:09
	;addfile "Software/Thomas/MINE-Bank/mine08_a000.bin.pck"	, 0A000h, , , 'M', "MINE 8 Raeume"
	addfile "Software/Thomas/MINE-Bank/mine74.bin.pck"	, 0A000h, 0E45Fh, , 'M', "MINE 74 Raeume"	; bis E45F !
	addfile "Software/volker/testbild.bin"			, 0E000h, , , 'B', "Test BASERW"

	;28.08.2021 11:24
	addfile "Software/Rolf/Bildbetrachter/Arthur.bmp.pck"	, 08000h, , , 'P', "ARTHUR"
	addfile "Software/Rolf/Bildbetrachter/pixelgrafik.bmp.pck", 08000h, , , 'P', "Pixelgrafik"

	addfile "Software/Rolf/Bildbetrachter/Buddel.bmp.pck"	, 08000h, , , 'P', "BUDDEL"
	addfile "Software/Rolf/Bildbetrachter/DDR.bmp.pck"	, 08000h, , , 'P', "DDR"
	
	;19.09.2021
	addfile "Software/Wolfgang2/Tropfen_PLUS.bin.pck"       , 0E000h, , , 'B', "Tropfen"
	addfile "Software/Wolfgang2/SCHMUGGLER_PLUS.bin.pck"    , 0E000h, , , 'B', "SCHMUGGLER"
	addfile "Software/Wolfgang2/Neun_PLUS.bin.pck"          , 0E000h, , , 'B', "NEUN GEWINNT"
	addfile "Software/Wolfgang2/INVASION_neu.bin.pck"       , 0E000h, , , 'B', "INVASION"

	; 19.11.2021 
	addfile "Software/Wolfgang2/Lauflicht_neu.bin.pck"      , 0E000h, , , 'B', "LAUFLICHT"

	;18.11.2021
	addfile "Software/volker/z8671basic_asm.bin.pck"	, 08000h, , , 'M', "BASIC/DEBUG"
	addfile "Software/Wolfgang2/BALL_neu.bin.pck"      	, 0E000h, , , 'B', "BALL"
	addfile "Software/Wolfgang2/PINGUIN.BIN.pck"      	, 0E000h, , , 'B', "PINGUIN"

	;16.02.2022
	addfile "Software/Thomas/OTHELLO-L8000.BIN.pck"		, 08000h, , , 'M', "OTHELLO"
	addfile "Software/Thomas/mine/DEMO.BIN.pck"		, 0A000h, , , 'M', "MINE DEMO"

	;21.03.2022
	addfile "Software/Rolf/FLOATEST.bin.pck"      		, 0E000h, , , 'B', "FLOAT-Test"

	;20.12.2022
	addfile "Software/volker/t2verb.bin.pck"      		, 0F100h, , , 'M', "TOR2-Verbindung"
;------------------------------------------------------------------------------

	if	$>=(lastbank+1)*blocksize
		WARNING "ROM-Kapazitt berschritten!"
	endif

        end
